"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const graphql_subscriptions_1 = require("graphql-subscriptions");
const nats_1 = require("nats");
const nats_pubsub_1 = require("../nats-pubsub");
const logger_1 = require("./logger");
require("jest");
const schema = new graphql_1.GraphQLSchema({
    query: new graphql_1.GraphQLObjectType({
        name: 'Query',
        fields: {
            testString: {
                type: graphql_1.GraphQLString,
                resolve: function () {
                    return 'works';
                },
            },
        },
    }),
    subscription: new graphql_1.GraphQLObjectType({
        name: 'Subscription',
        fields: {
            testSubscription: {
                type: graphql_1.GraphQLString,
                resolve: function (root) {
                    return root;
                },
            },
            testFilter: {
                type: graphql_1.GraphQLString,
                resolve: function (_, { filterBoolean }) {
                    return filterBoolean ? 'goodFilter' : 'badFilter';
                },
                args: {
                    filterBoolean: { type: graphql_1.GraphQLBoolean },
                },
            },
            testFilterMulti: {
                type: graphql_1.GraphQLString,
                resolve: function (_, { filterBoolean }) {
                    return filterBoolean ? 'goodFilter' : 'badFilter';
                },
                args: {
                    filterBoolean: { type: graphql_1.GraphQLBoolean },
                    a: { type: graphql_1.GraphQLString },
                    b: { type: graphql_1.GraphQLInt },
                },
            },
            testChannelOptions: {
                type: graphql_1.GraphQLString,
                resolve: function (root) {
                    return root;
                },
                args: {
                    repoName: { type: graphql_1.GraphQLString },
                },
            },
        },
    }),
});
const natsClient = nats_1.connect('nats://localhost:4222');
const subManager = new graphql_subscriptions_1.SubscriptionManager({
    schema,
    setupFunctions: {
        'testFilter': (_, { filterBoolean }) => {
            return {
                'Filter1': { filter: (root) => root.filterBoolean === filterBoolean },
            };
        },
        'testFilterMulti': () => {
            return {
                'Trigger1': { filter: () => true },
                'Trigger2': { filter: () => true },
            };
        },
    },
    pubsub: new nats_pubsub_1.NatsPubSub({
        client: natsClient,
        logger: logger_1.logger,
    }),
});
describe('SubscriptionManager', function () {
    let originalTimeout;
    beforeAll(function (done) {
        natsClient.on('connect', () => {
            done();
        });
    });
    afterAll(function (done) {
    });
    it('throws an error if query is not valid', () => __awaiter(this, void 0, void 0, function* () {
        const query = 'query a{ testInt }';
        const callback = () => null;
        return yield expect(subManager.subscribe({ query, operationName: 'a', callback }))
            .rejects.toMatchObject({ message: 'Subscription query has validation errors' });
    }));
    it('rejects subscriptions with more than one root field', () => __awaiter(this, void 0, void 0, function* () {
        const query = 'subscription X{ a: testSubscription, b: testSubscription }';
        const callback = () => null;
        return yield expect(subManager.subscribe({ query, operationName: 'X', callback }))
            .rejects.toMatchObject({ message: 'Subscription query has validation errors' });
    }));
    it('can subscribe with a valid query and gets a subId back', function (done) {
        const query = 'subscription X{ testSubscription }';
        const callback = () => null;
        subManager.subscribe({ query, operationName: 'X', callback }).then(subId => {
            expect(typeof subId).toBe('number');
            logger_1.logger.trace('subid (%s)', subId);
            try {
                subManager.unsubscribe(subId);
                done();
            }
            catch (e) {
                done.fail(e);
            }
        });
    });
    it('can subscribe with a nameless query and gets a subId back', function () {
        const query = 'subscription { testSubscription }';
        const callback = () => null;
        subManager.subscribe({ query, operationName: 'X', callback }).then(subId => {
            expect(typeof subId).toBe('number');
            subManager.unsubscribe(subId);
        });
    });
    it('can subscribe with a valid query and get the root value', (done) => {
        const query = 'subscription X{ testSubscription}';
        const callback = function (err, payload) {
            if (err) {
                done.fail(err);
            }
            logger_1.logger.trace('testOutput#4, payload (%j)', payload);
            try {
                expect(payload.data.testSubscription).toBe('good');
            }
            catch (e) {
                done.fail(e);
                return;
            }
            done();
        };
        subManager.subscribe({ query, operationName: 'X', callback }).then(subId => {
            subManager.publish('testSubscription', 'good');
            setTimeout(() => {
                subManager.unsubscribe(subId);
            }, 5);
        });
    });
    it('can use filter functions properly', function (done) {
        const query = `subscription Filter1($filterBoolean: Boolean){
            testFilter(filterBoolean: $filterBoolean)
            }`;
        const callback = function (err, payload) {
            if (err) {
                done.fail(err);
            }
            try {
                expect(payload.data.testFilter).toEqual('goodFilter');
            }
            catch (e) {
                done.fail(e);
                return;
            }
            done();
        };
        subManager.subscribe({
            query,
            operationName: 'Filter1',
            variables: { filterBoolean: true },
            callback,
        }).then(subId => {
            subManager.publish('Filter1', { filterBoolean: false });
            subManager.publish('Filter1', { filterBoolean: true });
            setTimeout(() => {
                subManager.unsubscribe(subId);
            }, 20);
        });
    });
    it('can use a filter function that returns a promise', function (done) {
        const query = `subscription Filter2($filterBoolean: Boolean){
       testFilter(filterBoolean: $filterBoolean)
      }`;
        const callback = function (err, payload) {
            if (err) {
                done.fail(err);
                return;
            }
            try {
                expect(payload.data.testFilter).toBe('goodFilter');
            }
            catch (e) {
                done.fail(e);
                return;
            }
            done();
        };
        subManager.subscribe({
            query,
            operationName: 'Filter2',
            variables: { filterBoolean: true },
            callback,
        }).then(subId => {
            subManager.publish('Filter2', { filterBoolean: false });
            subManager.publish('Filter2', { filterBoolean: true });
            subManager.unsubscribe(subId);
        });
    });
    it('can subscribe to more than one trigger', function (done) {
        let triggerCount = 0;
        const query = `subscription multiTrigger($filterBoolean: Boolean, $uga: String){
       testFilterMulti(filterBoolean: $filterBoolean, a: $uga, b: 66)
      }`;
        const callback = function (err, payload) {
            expect(payload.data.testFilterMulti).toBe('goodFilter');
            triggerCount++;
            if (triggerCount === 2) {
                done();
            }
        };
        subManager.subscribe({
            query,
            operationName: 'multiTrigger',
            variables: { filterBoolean: true, uga: 'UGA' },
            callback,
        }).then(subId => {
            subManager.publish('NotATrigger', { filterBoolean: false });
            subManager.publish('Trigger1', { filterBoolean: true });
            subManager.publish('Trigger2', { filterBoolean: true });
            setTimeout(() => {
                subManager.unsubscribe(subId);
            }, 6);
        });
    });
    it('can unsubscribe', function (done) {
        const query = 'subscription X{ testSubscription }';
        const callback = (err, payload) => {
            done();
        };
        subManager.subscribe({ query, operationName: 'X', callback }).then(subId => {
            subManager.unsubscribe(subId);
            subManager.publish('testSubscription', 'bad');
            setTimeout(done, 1000);
        });
    });
    it('throws an error when trying to unsubscribe from unknown id', function () {
        expect(() => subManager.unsubscribe(123)).toThrowError('undefined');
    });
    it('calls the error callback if there is an execution error', (done) => {
        const query = `subscription X($uga: Boolean!){
      testSubscription  @skip(if: $uga)
    }`;
        const callback = function (err, payload) {
            logger_1.logger.trace('test# received err (%j), payload (%j) ', err, payload);
            try {
                expect(payload).toBeDefined();
            }
            catch (e) {
                done.fail(e);
                return;
            }
            done();
        };
        subManager.subscribe({ query, operationName: 'X', callback }).then(subId => {
            subManager.publish('testSubscription', 'good');
            setTimeout(() => {
                subManager.unsubscribe(subId);
            }, 40);
        });
    });
    it('can use transform function to convert the trigger name given into more explicit channel name', function (done) {
        const triggerTransform = (trigger, { path }) => [trigger, ...path].join('.');
        const pubsub = new nats_pubsub_1.NatsPubSub({
            triggerTransform,
            logger: logger_1.logger,
        });
        const subManager2 = new graphql_subscriptions_1.SubscriptionManager({
            schema,
            setupFunctions: {
                testChannelOptions: (options, { repoName }) => ({
                    comments: {
                        channelOptions: { path: [repoName] },
                    },
                }),
            },
            pubsub,
        });
        const callback = (err, payload) => {
            expect(payload.data.testChannelOptions).toBe('test');
            done();
        };
        const query = `
      subscription X($repoName: String!) {
        testChannelOptions(repoName: $repoName)
      }
    `;
        const variables = { repoName: 'graphql-nats-subscriptions' };
        subManager2.subscribe({ query, operationName: 'X', variables, callback }).then(subId => {
            pubsub.publish('comments.graphql-nats-subscriptions', 'test');
            setTimeout(() => pubsub.unsubscribe(subId), 40);
        });
    });
});
//# sourceMappingURL=integration.test.js.map